@@ -1,26 +1,29 @@
 import random
 import string
+from dynaconf.utils.boxing import DynaBox
+from functools import cached_property
 from pathlib import Path
 
 import requests
 from logzero import logger
 
 from manifester.helpers import simple_retry
+from manifester.helpers import process_sat_version
 from manifester.logger import setup_logzero
 from manifester.settings import settings
 
 
 class Manifester:
     def __init__(self, manifest_category, allocation_name=None, **kwargs):
+        if isinstance(manifest_category, dict):
+            self.manifest_data = DynaBox(manifest_category)
+        else:
+            self.manifest_data = settings.manifest_category.get(manifest_category)
         self.allocation_name = allocation_name or "".join(
             random.sample(string.ascii_letters, 10)
         )
-        self.manifest_data = settings.manifest_category.get(manifest_category)
-        self.offline_token = kwargs.get(
-            "offline_token", self.manifest_data.get("offline_token", settings.offline_token)
-        )
+        self.offline_token = kwargs.get("offline_token", self.manifest_data.offline_token)
         self.subscription_data = self.manifest_data.subscription_data
-        self.sat_version = kwargs.get("sat_version", self.manifest_data.sat_version)
         self.token_request_data = {
             "grant_type": "refresh_token",
             "client_id": "rhsm-api",
@@ -29,8 +32,14 @@ class Manifester:
         self.simple_content_access = kwargs.get(
             "simple_content_access", self.manifest_data.simple_content_access
         )
+        self.token_request_url = self.manifest_data.get("url", {}).get("token_request", settings.url.token_request)
+        self.allocations_url = self.manifest_data.get("url", {}).get("allocations", settings.url.allocations)
         self._access_token = None
         self._subscription_pools = None
+        self.sat_version = process_sat_version(
+            kwargs.get("sat_version", self.manifest_data.sat_version),
+            self.valid_sat_versions,
+        )
 
     @property
     def access_token(self):
@@ -39,11 +48,29 @@ class Manifester:
             logger.debug("Generating access token")
             token_data = simple_retry(
                 requests.post,
-                cmd_args=[f"{self.manifest_data.url.token_request}"],
+                cmd_args=[f"{self.token_request_url}"],
                 cmd_kwargs=token_request_data,
             ).json()
             self._access_token = token_data["access_token"]
         return self._access_token
+    
+    @cached_property
+    def valid_sat_versions(self):
+        headers = {
+            "headers": {"Authorization": f"Bearer {self.access_token}"},
+            "proxies": self.manifest_data.get("proxies", settings.proxies),
+        }
+        valid_sat_versions = []
+        sat_versions_response = simple_retry(
+            requests.get,
+            cmd_args=[
+                    f"{self.allocations_url}/versions"
+                ],
+            cmd_kwargs=headers,
+            ).json()
+        for ver_dict in sat_versions_response["body"]:
+            valid_sat_versions.append(ver_dict["value"])
+        return valid_sat_versions
 
     def create_subscription_allocation(self):
         allocation_data = {
@@ -57,10 +84,30 @@ class Manifester:
         }
         self.allocation = simple_retry(
             requests.post,
-            cmd_args=[f"{self.manifest_data.url.allocations}"],
+            cmd_args=[f"{self.allocations_url}"],
             cmd_kwargs=allocation_data,
         ).json()
+        logger.debug(
+            f"Received response {self.allocation} when attempting to create allocation."
+        )
+        if ("error" in self.allocation.keys() and 
+            "invalid version" in self.allocation['error'].values()):
+            raise ValueError(
+                                f"{self.sat_version} is not a valid version number."
+                                "Versions must be in the form of \"sat-X.Y\". Current"
+                                f"valid versions are {self.valid_sat_versions}."
+                            )
         self.allocation_uuid = self.allocation["body"]["uuid"]
+        if self.simple_content_access == "disabled":
+            simple_retry(
+                requests.put,
+                cmd_args=[f"{self.allocations_url}/{self.allocation_uuid}"],
+                cmd_kwargs={
+                    "headers": {"Authorization": f"Bearer {self.access_token}"},
+                    "proxies": self.manifest_data.get("proxies", settings.proxies),
+                    "json": {"simpleContentAccess": "disabled"},
+                },
+            )
         logger.info(
             f"Subscription allocation created with name {self.allocation_name} "
             f"and UUID {self.allocation_uuid}"
@@ -76,7 +123,7 @@ class Manifester:
         }
         response = simple_retry(
             requests.delete,
-            cmd_args=[f"{self.manifest_data.url.allocations}/{self.allocation_uuid}"],
+            cmd_args=[f"{self.allocations_url}/{self.allocation_uuid}"],
             cmd_kwargs=data,
         )
         return response
@@ -93,7 +140,7 @@ class Manifester:
             self._subscription_pools = simple_retry(
                 requests.get,
                 cmd_args=[
-                    f"{self.manifest_data.url.allocations}/{self.allocation_uuid}/pools"
+                    f"{self.allocations_url}/{self.allocation_uuid}/pools"
                 ],
                 cmd_kwargs=data,
             ).json()
@@ -115,7 +162,7 @@ class Manifester:
                 offset_pools = simple_retry(
                     requests.get,
                     cmd_args=[
-                        f"{self.manifest_data.url.allocations}/{self.allocation_uuid}/pools"
+                        f"{self.allocations_url}/{self.allocation_uuid}/pools"
                     ],
                     cmd_kwargs=data,
                 ).json()
@@ -136,7 +183,7 @@ class Manifester:
         add_entitlements = simple_retry(
             requests.post,
             cmd_args=[
-                f"{self.manifest_data.url.allocations}/{self.allocation_uuid}/entitlements"
+                f"{self.allocations_url}/{self.allocation_uuid}/entitlements"
             ],
             cmd_kwargs=data,
         )
@@ -153,7 +200,7 @@ class Manifester:
         }
         self.entitlement_data = simple_retry(
             requests.get,
-            cmd_args=[f"{self.manifest_data.url.allocation}/{self.allocation_uuid}"],
+            cmd_args=[f"{self.allocations_url}/{self.allocation_uuid}"],
             cmd_kwargs=data,
         ).json()
         current_entitlement = [
@@ -193,7 +240,8 @@ class Manifester:
             f"The following pools are matches for this subscription: {matching}"
         )
         for match in matching:
-            if match["entitlementsAvailable"] > subscription_data["quantity"]:
+            if (match["entitlementsAvailable"] > subscription_data["quantity"] or
+                match["entitlementsAvailable"] == -1):
                 logger.debug(
                     f"Pool {match['id']} is a match for this subscription and has "
                     f"{match['entitlementsAvailable']} entitlements available."
@@ -266,7 +314,7 @@ class Manifester:
         trigger_export_job = simple_retry(
             requests.get,
             cmd_args=[
-                f"{self.manifest_data.url.allocations}/{self.allocation_uuid}/export"
+                f"{self.allocations_url}/{self.allocation_uuid}/export"
             ],
             cmd_kwargs=data,
         ).json()
@@ -274,7 +322,7 @@ class Manifester:
         export_job = simple_retry(
             requests.get,
             cmd_args=[
-                f"{self.manifest_data.url.allocations}/{self.allocation_uuid}/exportJob/{export_job_id}"
+                f"{self.allocations_url}/{self.allocation_uuid}/exportJob/{export_job_id}"
             ],
             cmd_kwargs=data,
         )
@@ -284,7 +332,7 @@ class Manifester:
             export_job = simple_retry(
                 requests.get,
                 cmd_args=[
-                    f"{self.manifest_data.url.allocations}/{self.allocation_uuid}/exportJob/{export_job_id}"
+                    f"{self.allocations_url}/{self.allocation_uuid}/exportJob/{export_job_id}"
                 ],
                 cmd_kwargs=data,
             )
@@ -313,6 +361,7 @@ class Manifester:
             f"{local_file}"
         )
         local_file.write_bytes(manifest.content)
+        manifest.path = local_file
         return manifest
 
     def get_manifest(self):
